<?php namespace Myriad\Installations\Controllers;

use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Myriad\Installations\Exceptions\InstallationValidationException;
use Myriad\Installations\InstallationManager;
use Myriad\Installations\Models\Installation;
use Myriad\Installations\Validation\IInstallationValidationProvider;

/**
 * Class InstallationsController accepts HTTP requests for CRUD operations on installations.
 *
 * @package Myriad\Installations\Controllers
 */
class InstallationsController extends Controller {
  
  /**
   * @var InstallationManager knows how to operate on installations.
   */
  private $installationManager;
  /**
   * @var IInstallationValidationProvider knows how to operate on installations.
   */
  private $installationValidationProvider;

  /**
   * InstallationsController constructor.
   *
   * @param InstallationManager $installationManager knows how to operate on installations.
   * @param IInstallationValidationProvider $installationValidationProvider knows how to validation installations.
   */
  public function __construct(InstallationManager $installationManager, IInstallationValidationProvider $installationValidationProvider) {
    $this->installationManager = $installationManager;
    $this->installationValidationProvider = $installationValidationProvider;
  }

  /**
   * Finds an Installation by id.
   *
   * @param string $uuid The UUID of the installation.
   * @param Request $request The contents of the HTTP request.
   * @return Response
   */
  public function show($uuid, Request $request) {
    try {
      $this->installationValidationProvider->validateUUID($uuid);
    } catch (InstallationValidationException $e) {
      return $this->renderException($request, $e);
    }

    $installation = $this->installationManager->findInstallationByUUID($uuid);

    if (!$installation) {
      $mnfe = new ModelNotFoundException();
      $mnfe->setModel(Installation::class);
      return $this->renderException($request, $mnfe);
    }

    return $this->renderShow($request, $installation);
  }

  /**
   * Creates a new Installation.
   *
   * @param string $uuid The UUID of the installation.
   * @param Request $request The contents of the HTTP request.
   * @return Response
   */
  public function store($uuid, Request $request) {
    try {
      $this->installationValidationProvider->validateUUID($uuid);
    } catch (InstallationValidationException $e) {
      return $this->renderException($request, $e);
    }

    $data = array_merge($this->installationManager->getParamsFromRequest(), $request->all());
    $data['uuid'] = $uuid;

    $validator = $this->installationValidationProvider->getValidator($data);
    if ($validator->fails()) {
      return $this->renderException($request, new InstallationValidationException($validator));
    }

    $installation = $this->installationManager->createOrUpdateInstallation($uuid, $data);

    return $this->renderShow($request, $installation);
  }

  /**
   * Renders successful responses from the show() method.
   *
   * @param Request $request The contents of the HTTP request.
   * @param Installation $installation The installation to create a response for.
   * @return JsonResponse
   */
  public function renderShow(Request $request, Installation $installation) {
    return $this->renderInstallation($installation);
  }

  /**
   * Renders successful responses from the store() method.
   *
   * @param Request $request The contents of the HTTP request.
   * @param Installation $installation The installation go to create a response for.
   * @return JsonResponse
   */
  public function renderStore(Request $request, Installation $installation) {
    return $this->renderInstallation($installation);
  }

  /**
   * Renders an installation as JSON.
   *
   * @param Installation $installation The installation to create a response for.
   * @return JsonResponse
   */
  public function renderInstallation(Installation $installation) {
    return new JsonResponse(['data' => $installation]);
  }

  /**
   * Renders exception responses.
   *
   * @param Request $request The contents of the HTTP request.
   * @param \Exception $exception
   * @return null
   * @throws \Exception
   */
  public function renderException(Request $request, \Exception $exception) {
    throw $exception;
  }

}