<?php namespace Myriad\Installations\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * Class Installation represents an Installation. It provides access to the database representation of one or many Installation objects. It also knows how to apply business rules to Installation objects.
 *
 * @property string id The unique auto-incrementing key of the Installation generated by the database.
 * @property string uuid The UUID of the Installation generated by the device the installation represents.
 * @property string app_name The name of the app.
 * @property string app_version The version of the app.
 * @property string platform_name The name of the platform.
 * @property string platform_version The version of the platform.
 * @property string locale The locale the device is set to.
 * @property string timezone The timezone the device is set to.
 * @property string push_type The push platform.
 * @property string push_id The ID the push platform uses to identify the device.
 * @property string user_agent The value of the user agent header the device sends when making requests.
 * @property array custom An array with a bunch of custom values the application needs.
 * @property Carbon created_at The time when the Installation was created.
 * @property Carbon updated_at The time when the Installation was lasted updated.
 * @property Carbon deleted_at The time when the Installation was deleted(used by soft deletes).
 */
class Installation extends Model {

  use SoftDeletes;

  /**
   * The attributes that aren't mass assignable.
   *
   * @var array
   */
  protected $guarded = ['id', 'uuid', 'custom', 'created_at', 'updated_at', 'deleted_at'];

  /**
   * The attributes that should be hidden for arrays.
   *
   * @var array
   */
  protected $hidden = ['id', 'custom'];

  /**
   * The attributes that should be casted to native types.
   *
   * @var array
   */
  protected $casts = [
    'custom' => 'json',
  ];

  /**
   * Get the table associated with the model.
   *
   * @return string
   */
  public function getTable() {
    return config('installations.table_name', 'installations');
  }

  /**
   * Put a key value pair to custom.
   *
   * @param string $key The key of the custom value.
   * @param string $value The value associated with the key.
   */
  public function putCustom($key, $value) {
    $this->custom = array_merge(is_array($this->custom) ? $this->custom : [], [$key => $value]);
  }

  /**
   * Get a value from custom.
   *
   * @param string $key The key to get.
   * @param mixed $default The value to return if the key doesn't exist.
   * @return mixed
   */
  public function getCustom($key, $default) {
    $custom = $this->custom;
    if (!is_array($custom)) {
      return $default;
    }
    return array_key_exists($key, $custom) ? $custom[$key] : $default;
  }

}