<?php namespace Myriad\Installations\Validation;

use Illuminate\Contracts\Validation\Factory;
use Illuminate\Support\Arr;
use Myriad\Installations\Exceptions\InstallationValidationException;
use Ramsey\Uuid\Uuid;

/**
 * Class InstallationValidationProvider validates installations.
 *
 * @package Myriad\Installations\Validation
 */
class InstallationValidationProvider implements IInstallationValidationProvider {

  /**
   * @var Factory creates validators.
   */
  private $validator;

  /**
   * InstallationValidationProvider constructor.
   *
   * @param Factory $validator creates validators.
   */
  public function __construct(Factory $validator) {
    $this->validator = $validator;
  }

  /**
   * @inheritdoc
   */
  public function getRules($data = []) {
    return [
      'uuid' => 'required|uuid',
      'app_name' => 'required|max:64',
      'app_version' => 'required|max:64',
      'platform_name' => 'in:android,ios,web',
      'platform_version' => 'max:32',
      'locale' => 'locale',
      'timezone' => 'timezone',
      'push_type' => 'in:parse,gcm,apns',
      'push_id' => 'max:255',
      'user_agent' => 'required'
    ];
  }

  /**
   * @inheritdoc
   */
  public function getMessages($data = []) {
    return [];
  }

  /**
   * @inheritdoc
   */
  public function getValidator($data) {
    return $this->validator->make($data, $this->getRules($data), $this->getMessages($data));
  }

  /**
   * @inheritdoc
   */
  public function validateUUID($uuid) {
    $uuidRules = Arr::get($this->getRules(['uuid' => $uuid]), 'uuid');
    $uuidMessages = Arr::get($this->getMessages(['uuid' => $uuid]), 'uuid', []);
    if (!empty($uuidRules)) {
      $validator = $this->validator->make(['uuid' => $uuid], ['uuid' => $uuidRules], $uuidMessages);
      if ($validator->fails()) {
        throw new InstallationValidationException($validator);
      }
    }
    return true;
  }

  /**
   * @inheritdoc
   */
  public function normalizeUUID($uuid) {
    try {
      return strtolower(Uuid::fromString($uuid)->toString());
    } catch (\Exception $e) {
      return $uuid;
    }
  }

}
